package api

import (
	"errors"
	"moutai/log"
	"time"

	"moutai/db/model"
	"moutai/db/repo"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func RoleCreate(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	var form model.Role
	err := c.ShouldBindJSON(&form)
	if err != nil {
		return nil, err
	}
	form.CreateTime = time.Now()
	form.UpdateTime = time.Now()
	return repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionRoles, &form)
}

func DeleteRole(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	_id := c.Param("id")
	id, _ := primitive.ObjectIDFromHex(_id)
	if id.IsZero() {
		return nil, errors.New("id错误")
	}
	return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionRoles, _id)

}

func RoleList(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	page, size, query := UtilQueryPageSize(c)

	return repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
		CollectName: repo.CollectionRoles,
		Page:        page,
		Size:        size,
		Query:       query,
	})

}

func UpdateRole(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	role := &model.Role{}
	err := c.ShouldBindJSON(&role)
	if err != nil {
		log.Error(err)
		return nil, err
	}
	if role.Id.IsZero() {
		return nil, errors.New("id错误")
	}

	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionUser, role.Id.Hex(), role)

}
