package api

import (
	"errors"
	"moutai/db/model"
	"moutai/db/repo"
	"moutai/log"
	"strconv"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func CreateVideo(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	Video := &model.Videos{}
	err := c.ShouldBindJSON(&Video)
	if err != nil {
		log.Error(err)
		return nil, err
	}

	// 正常状态
	state := -1
	Video.State = &state
	Video.CreateTime = time.Now()
	Video.UpdateTime = time.Now()
	return repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionVideos, &Video)
}

func DeleteVideo(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	_id := c.Param("id")
	id, _ := primitive.ObjectIDFromHex(_id)
	if id.IsZero() {
		return nil, errors.New("id错误")
	}
	return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionVideos, _id)
}

func VideoList(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	page, size, query := UtilQueryPageSize(c)
	_uploadTime := c.Query("uploadTime")
	_cid := c.Query("cid")
	cid, _ := primitive.ObjectIDFromHex(_cid)
	name := c.Query("name")
	_sort := c.Query("sort")
	sortInt, _ := strconv.Atoi(_sort)
	sort := repo.Map{"createTime": -1}
	if sortInt > 0 {
		sort = repo.Map{"sort": -1}
	}

	if !cid.IsZero() {
		query["cid"] = cid
	}
	if len(name) > 0 {
		query["name"] = bson.M{"$regex": name, "$options": "$i"}
	}
	if len(_uploadTime) > 0 {
		uploadTime, err := time.Parse("2006-01-02", _uploadTime)
		if err == nil {
			startTime := time.Date(uploadTime.Year(), uploadTime.Month(), uploadTime.Day(), 0, 0, 0, 0, uploadTime.Location())
			endTime := time.Date(uploadTime.Year(), uploadTime.Month(), uploadTime.Day(), 23, 59, 59, 0, uploadTime.Location())
			query["createTime"] = bson.M{
				"$gte": startTime,
				"$lte": endTime,
			}
		}
	}

	result, err := repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
		CollectName: repo.CollectionVideos,
		Page:        page,
		Size:        size,
		Query:       query,
		Sort:        sort,
	})

	cate := model.Catgory{}
	if len(result.List) > 0 {
		for i, u := range result.List {
			if v, ok := u["cid"]; ok {
				// 查询角色信息
				cid := v.(primitive.ObjectID)
				repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
					CollectName: repo.CollectionCategories,
					Query:       repo.Map{"_id": cid},
				}, &cate)
			}
			// 拼接角色信息
			result.List[i]["cateInfo"] = cate
		}
	}
	return result, err
}

func UpdateVideo(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	Video := &model.Videos{}
	err := c.ShouldBindJSON(&Video)
	if err != nil {
		log.Error(err)
		return nil, err
	}
	if Video.Id.IsZero() {
		return nil, errors.New("id错误")
	}

	Video.UpdateTime = time.Now()
	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionVideos, Video.Id.Hex(), Video)
}

func VideoOn(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	_id := c.Param("id")
	id, _ := primitive.ObjectIDFromHex(_id)
	if id.IsZero() {
		return nil, errors.New("id错误")
	}
	state := 1
	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionVideos, _id, model.Videos{State: &state})
}

func VideoOff(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	_id := c.Param("id")
	id, _ := primitive.ObjectIDFromHex(_id)
	if id.IsZero() {
		return nil, errors.New("id错误")
	}
	state := -1
	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionVideos, _id, model.Videos{State: &state})
}
